'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

function _interopNamespace(e) {
  if (e && e.__esModule) { return e; } else {
    var n = {};
    if (e) {
      Object.keys(e).forEach(function (k) {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () {
            return e[k];
          }
        });
      });
    }
    n['default'] = e;
    return n;
  }
}

var tslib = require('tslib');
var defu = _interopDefault(require('defu'));
var getPort = _interopDefault(require('get-port'));
var path = require('path');

expect.extend({
  toNuxtPluginAdded: function toNuxtPluginAdded(ctx, plugin) {
    expect(ctx.nuxt.moduleContainer.addPlugin).toBeCalledWith(plugin);
    return {
      pass: true,
      message: function message() {
        return '';
      }
    };
  },
  toNuxtLayoutAdded: function toNuxtLayoutAdded(ctx, layout, name) {
    if (name) {
      expect(ctx.nuxt.moduleContainer.addLayout).toBeCalledWith(layout, name);
    } else {
      expect(ctx.nuxt.moduleContainer.addLayout).toBeCalledWith(layout);
    }

    return {
      pass: true,
      message: function message() {
        return '';
      }
    };
  },
  toNuxtErrorLayoutAdded: function toNuxtErrorLayoutAdded(ctx, errorLayout) {
    expect(ctx.nuxt.moduleContainer.addErrorLayout).toBeCalledWith(errorLayout);
    return {
      pass: true,
      message: function message() {
        return '';
      }
    };
  },
  toNuxtServerMiddlewareAdded: function toNuxtServerMiddlewareAdded(ctx, middleware) {
    expect(ctx.nuxt.moduleContainer.addServerMiddleware).toBeCalledWith(middleware);
    return {
      pass: true,
      message: function message() {
        return '';
      }
    };
  },
  toNuxtRequireModule: function toNuxtRequireModule(ctx, moduleOpts) {
    expect(ctx.nuxt.moduleContainer.requireModule).toBeCalledWith(moduleOpts);
    return {
      pass: true,
      message: function message() {
        return '';
      }
    };
  }
});

var currentContext;
function createContext(options) {
  return setContext(defu(options, {
    __dirname: __dirname,
    configFile: 'nuxt.config.js',
    browserString: 'puppeteer',
    buildTimeout: 60000,
    server: options.browser,
    build: options.browser || options.server,
    config: {}
  }));
}
function getContext() {
  if (!currentContext) {
    throw new Error('No context is available. (Forgot calling setup or createContext?)');
  }

  return currentContext;
}
function setContext(context) {
  currentContext = context;
  return currentContext;
}

function listen() {
  return tslib.__awaiter(this, void 0, void 0, function* () {
    var ctx = getContext();
    var port = yield getPort();
    ctx.url = 'http://localhost:' + port;
    yield ctx.nuxt.listen(port);
  });
}
function url(path) {
  var ctx = getContext();
  return ctx.url + path;
}

function createBrowser() {
  return tslib.__awaiter(this, void 0, void 0, function* () {
    var ctx = getContext();
    var tib = yield Promise.resolve().then(function () { return _interopNamespace(require('tib')); });
    ctx.browser = yield tib.createBrowser(ctx.browserString, ctx.browserOptions);
  });
}
function createPage(path) {
  return tslib.__awaiter(this, void 0, void 0, function* () {
    var ctx = getContext();
    var page = yield ctx.browser.page(url(path));
    return page;
  });
}

function loadNuxt() {
  return tslib.__awaiter(this, void 0, void 0, function* () {
    var ctx = getContext();

    var _yield$loadNuxtPackag = yield loadNuxtPackage(),
        Nuxt = _yield$loadNuxtPackag.Nuxt;

    ctx.nuxt = new Nuxt(ctx.config);
  });
}
function loadFixture() {
  return tslib.__awaiter(this, void 0, void 0, function* () {
    var ctx = getContext();
    ctx.rootDir = path.resolve(ctx.__dirname, ctx.fixture);
    var loadedConfig = yield Promise.resolve().then(function () { return _interopNamespace(require(path.resolve(ctx.rootDir, ctx.configFile))); }).then(function (m) {
      return (
        /* istanbul ignore next */
        m["default"] || m
      );
    });
    ctx.config = defu(ctx.config, loadedConfig);

    if (!ctx.config.rootDir) {
      ctx.config.rootDir = ctx.rootDir;
    }
  });
}
function loadNuxtPackage() {
  var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'nuxt';
  return tslib.__awaiter(this, void 0, void 0, function* () {
    return yield Promise.resolve().then(function () { return _interopNamespace(require(name + '-edge')); })["catch"](
    /* istanbul ignore next */
    function () {
      return Promise.resolve().then(function () { return _interopNamespace(require(name)); });
    });
  });
}

function build() {
  return tslib.__awaiter(this, void 0, void 0, function* () {
    var ctx = getContext();

    var _yield$loadNuxtPackag = yield loadNuxtPackage(),
        Builder = _yield$loadNuxtPackag.Builder;

    ctx.builder = new Builder(ctx.nuxt);
    yield ctx.builder.build();
  });
}

function generate() {
  return tslib.__awaiter(this, void 0, void 0, function* () {
    var ctx = getContext();

    var _yield$loadNuxtPackag = yield loadNuxtPackage(),
        Builder = _yield$loadNuxtPackag.Builder,
        Generator = _yield$loadNuxtPackag.Generator;

    var builder = new Builder(ctx.nuxt);
    var generator = new Generator(ctx.nuxt, builder);
    yield generator.generate(ctx.generateOptions);
  });
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelper(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = o[Symbol.iterator]();
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

function setupTest(options) {
  var _this = this;

  var ctx = createContext(options);
  beforeAll(function () {
    return tslib.__awaiter(_this, void 0, void 0, function* () {
      if (ctx.fixture) {
        yield loadFixture();
      }

      if (!ctx.nuxt) {
        yield loadNuxt();
        spyOnClass(ctx.nuxt.moduleContainer);
        yield ctx.nuxt.ready();
      }

      if (ctx.build && !ctx.builder) {
        yield build();
      }

      if (ctx.generate) {
        yield generate();
      }

      if (ctx.waitFor) {
        yield new Promise(function (resolve) {
          return setTimeout(resolve, ctx.waitFor);
        });
      }

      if (ctx.server && !ctx.url) {
        yield listen();
      }

      if (ctx.browser === true) {
        yield createBrowser();
      }
    });
  }, ctx.buildTimeout);
  afterAll(function () {
    return tslib.__awaiter(_this, void 0, void 0, function* () {
      yield cleanup();
    });
  });
  return ctx;
}
function cleanup() {
  return tslib.__awaiter(this, void 0, void 0, function* () {
    var ctx = getContext();

    if (ctx.nuxt) {
      yield ctx.nuxt.close();
    }

    if (ctx.browser) {
      yield ctx.browser.close();
    }
  });
}
function spyOnClass(instance) {
  var proto = Object.getPrototypeOf(instance);

  var _iterator = _createForOfIteratorHelper(Object.getOwnPropertyNames(proto)),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var key = _step.value;
      jest.spyOn(instance, key);
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
}

exports.build = build;
exports.cleanup = cleanup;
exports.createBrowser = createBrowser;
exports.createContext = createContext;
exports.createPage = createPage;
exports.generate = generate;
exports.getContext = getContext;
exports.listen = listen;
exports.loadFixture = loadFixture;
exports.loadNuxt = loadNuxt;
exports.loadNuxtPackage = loadNuxtPackage;
exports.setContext = setContext;
exports.setupTest = setupTest;
exports.spyOnClass = spyOnClass;
exports.url = url;
